#
# $Id: Plugin.pm 2186 2012-10-23 13:12:32Z gomor $
#
package Net::SinFP3::Plugin;
use strict;
use warnings;

1;

__END__

=head1 NAME

Net::SinFP3::Plugin - how to create a SinFP3 plugin

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 ATTRIBUTES

=head1 METHODS

=head1 AUTHOR

Patrice E<lt>GomoRE<gt> Auffret

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2011-2012, Patrice E<lt>GomoRE<gt> Auffret

You may distribute this module under the terms of the Artistic license.
See LICENSE.Artistic file in the source distribution archive.

=cut
